mod address;
mod buddy_system_allocator;
mod frame_allocator;
mod linked_list;
mod memory_set;
mod page_table;
mod shm;

use address::VPNRange;
pub use address::{PhysAddr, PhysPageNum, StepByOne, VirtAddr, VirtPageNum};
use alloc::collections::BTreeMap;
use alloc::string::String;
use buddy_system_allocator::MEMORY_ALLOCATOR;
pub use frame_allocator::{frame_alloc, frame_dealloc, frames_alloc, frames_dealloc, FrameTracker};
pub use memory_set::remap_test;
pub use memory_set::{kernel_token, MapPermission, MapType, MemorySet, KERNEL_SPACE};
use page_table::PTEFlags;
pub use page_table::{
    get_data, put_data, translated_byte_buffer, translated_ref, translated_refmut, translated_str,
    PageTable, PageTableEntry, UserBuffer, UserBufferIterator,
};
pub use shm::{shm_attach, shm_create, shm_find};

pub fn init() {
    buddy_system_allocator::init_memory_allocator();
}

pub fn activate() {
    KERNEL_SPACE.activate();
}

pub fn get_meminfo() -> BTreeMap<String, usize> {
    // 根据 busybox 源码 busybox/procps/free.c 添加
    let mut res = BTreeMap::new();
    res.insert(String::from("MemAvailable"), {
        let allocator = MEMORY_ALLOCATOR.lock();
        (allocator.total - allocator.allocated) / 1024
    });
    res.insert(String::from("Cached"), 0);
    res.insert(String::from("SReclaimable"), 0);
    res
}
