use bitflags::*;

pub const MAX_SIG: usize = 31;

bitflags! {
    pub struct SignalFlags: u64 {
        const SIGHUP = 1 << (1 - 1);
        const SIGINT = 1 << (2 - 1);
        const SIGQUIT = 1 << (3 - 1);
        const SIGILL = 1 << (4 - 1);
        const SIGTRAP = 1 << (5 - 1);
        const SIGABRT = 1 << (6 - 1);
        const SIGBUS = 1 << (7 - 1);
        const SIGFPE = 1 << (8 - 1);
        const SIGKILL = 1 << (9 - 1);
        const SIGUSR1 = 1 << (10 - 1);
        const SIGSEGV = 1 << (11 - 1);
        const SIGUSR2 = 1 << (12 - 1);
        const SIGPIPE = 1 << (13 - 1);
        const SIGALRM = 1 << (14 - 1);
        const SIGTERM = 1 << (15 - 1);
        const SIGSTKFLT = 1 << (16 - 1);
        const SIGCHLD = 1 << (17 - 1);
        const SIGCONT = 1 << (18 - 1);
        const SIGSTOP = 1 << (19 - 1);
        const SIGTSTP = 1 << (20 - 1);
        const SIGTTIN = 1 << (21 - 1);
        const SIGTTOU = 1 << (22 - 1);
        const SIGURG = 1 << (23 - 1);
        const SIGXCPU = 1 << (24 - 1);
        const SIGXFSZ = 1 << (25 - 1);
        const SIGVTALRM = 1 << (26 - 1);
        const SIGPROF = 1 << (27 - 1);
        const SIGWINCH = 1 << (28 - 1);
        const SIGIO = 1 << (29 - 1);
        const SIGPWR = 1 << (30 - 1);
        const SIGSYS = 1 << (31 - 1);
    }
}

impl SignalFlags {
    pub fn from_sig(signum: usize) -> Self {
        Self::from_bits(1 << (signum - 1)).unwrap()
    }

    pub fn default_operation(&self) -> SignalOperation {
        let terminate_signals = SignalFlags::SIGHUP
            | SignalFlags::SIGINT
            | SignalFlags::SIGKILL
            | SignalFlags::SIGUSR1
            | SignalFlags::SIGUSR2
            | SignalFlags::SIGPIPE
            | SignalFlags::SIGALRM
            | SignalFlags::SIGTERM
            | SignalFlags::SIGSTKFLT
            | SignalFlags::SIGVTALRM
            | SignalFlags::SIGPROF
            | SignalFlags::SIGIO
            | SignalFlags::SIGPWR;
        let dump_signals = SignalFlags::SIGQUIT
            | SignalFlags::SIGILL
            | SignalFlags::SIGTRAP
            | SignalFlags::SIGABRT
            | SignalFlags::SIGBUS
            | SignalFlags::SIGFPE
            | SignalFlags::SIGSEGV
            | SignalFlags::SIGXCPU
            | SignalFlags::SIGXFSZ
            | SignalFlags::SIGSYS;
        let ignore_signals = SignalFlags::SIGCHLD | SignalFlags::SIGURG | SignalFlags::SIGWINCH;
        let stop_signals = SignalFlags::SIGSTOP
            | SignalFlags::SIGTSTP
            | SignalFlags::SIGTTIN
            | SignalFlags::SIGTTOU;
        let continue_signals = SignalFlags::SIGCONT;
        if terminate_signals.contains(*self) {
            SignalOperation::Terminate
        } else if dump_signals.contains(*self) {
            SignalOperation::Dump
        } else if ignore_signals.contains(*self) {
            SignalOperation::Ignore
        } else if stop_signals.contains(*self) {
            SignalOperation::Stop
        } else if continue_signals.contains(*self) {
            SignalOperation::Continue
        } else {
            println!("[kernel] signal {:?}: undefined default operation", self);
            SignalOperation::Terminate
        }
    }
}

#[derive(Clone, Copy)]
pub enum SignalOperation {
    Terminate,
    Dump,
    Ignore,
    Stop,
    Continue,
}
