use super::Inode;
use alloc::sync::Arc;
use spin::{Mutex, MutexGuard};

pub struct FSInfo {
    inner: Mutex<FSInfoInner>,
}

pub struct FSInfoInner {
    /// 当前工作目录
    pub cwd: Arc<dyn Inode>,
}

impl FSInfo {
    pub fn inner_exclusive_access(&self) -> MutexGuard<'_, FSInfoInner> {
        self.inner.lock()
    }

    pub fn new(cwd: Arc<dyn Inode>) -> Self {
        Self {
            inner: Mutex::new(FSInfoInner { cwd }),
        }
    }

    pub fn clone(&self) -> Self {
        let inner = self.inner_exclusive_access();
        Self {
            inner: Mutex::new(FSInfoInner {
                cwd: Arc::clone(&inner.cwd),
            }),
        }
    }
}
